// savi_setup.js

// https://savi-cdn.macmillantech.com/brightcove/savi-brightcove-player.js

// polyfill
Number.isInteger = Number.isInteger || function(value) {
    return typeof value === 'number' && 
      isFinite(value) && 
      Math.floor(value) === value;
};

// make sure safe_log is defined
if (typeof safe_log !== 'function') {
    var safe_log = function(message) {
        try {
            console.log(message);
        } 
        catch(e) {}
    }
}

// Add CSS file for SAVI player
(function () {
    const style = document.createElement('link');
    style.type = 'text/css';
    style.rel = 'stylesheet';
    style.href = "https://savi-cdn.macmillantech.com/fonts/SourceSansPro/sourceSansPro.css";
    document.getElementsByTagName('head')[0].appendChild(style);
})();

// Convert "mov" figures to SAVI divs
(function () {
    $('[data-mmtype="mov"]').each(function() {
        const $img = $(this);
        const bc_id = $img.attr('data-mmsrc');
        //var height = $img.attr('height');
        //var width = $img.attr('width');
        const id = $img.attr('id');

        $img.parent().replaceWith('<div id="' + id + '" class="savi-brightcove" ' + 'data-videoid="' + bc_id + '" data-suppresstitleoverwrite="true" ' + 
        'style="width: 100%"></div>');
    });
})();

class SaviBcVideo {

    constructor(id) {
        this.bc_id = id;
        this.video_watched = false;
        this.video_player = undefined;
        // the actual <video> element
        this.video_element = undefined;
        // track if video is currently in PiP window
        this.inPiPWindow = false; 
    }

    get id() {
        return String(this.bc_id);
    }
    
    get watched() {
        return this.video_watched;
    }

    set watched(status) {
        if (status !== true) {
            status = false;
        }
        this.video_watched = status;
    }

    get player() {
        return this.video_player;
    }

    add_video_player() {
        const context = this;
        videojs('player_'+this.bc_id).ready(function() {
            context.video_player = this;
            context._get_video_element();
            context._add_ended_callback();
            context._add_play_callback();
            context._add_pause_callback();
            context._add_pip_listeners();
        });
    }

    _get_video_element() {
        const children = this.video_player.children_;
        if (children === undefined) {
            return;
        } 
        for (let i = 0; i < children.length; i++) {
            if (children[i].tagName.toLowerCase() === 'video') {
                this.video_element = children[i];
                break;
            }
        }
    }

    _add_pip_listeners() {
        // Chrome and Edge implement the PiP API right now,
        // Firefox does not
        if (!('pictureInPictureEnabled' in document || !document.pictureInPictureEnabled)) {
            return;
        }

        // Not sure which is the best element to add
        // these to at the moment, try <video> first
        if (this.video_element === undefined) {
            this.video_element.addEventListener('enterpictureinpicture', () => {
                this.inPiPWindow = true;

                // call custom event handlers if defined
                if (typeof(SaviBcHandlers['enterpictureinpicture_'+this.bc_id]) === 'function') {
                    SaviBcHandlers['enterpictureinpicture_'+this.bc_id](this);
                }
                else if (typeof SaviBcHandlers.enterpictureinpicture_default === 'function') {
                    SaviBcHandlers.enterpictureinpicture_default(this);
                }
            });
            this.video_element.addEventListener('leavepictureinpicture', () => {
                this.inPiPWindow = false;

                // call custom event handlers if defined
                if (typeof(SaviBcHandlers['leavepictureinpicture_'+this.bc_id]) === 'function') {
                    SaviBcHandlers['leavepictureinpicture_'+this.bc_id](this);
                }
                else if (typeof SaviBcHandlers.leavepictureinpicture_default === 'function') {
                    SaviBcHandlers.leavepictureinpicture_default(this);
                }
            });
        }
        else {
            this.video_player.on('enterpictureinpicture', () => {
                this.inPiPWindow = true;

                // call custom event handlers if defined
                if (typeof(SaviBcHandlers['enterpictureinpicture_'+this.bc_id]) === 'function') {
                    SaviBcHandlers['enterpictureinpicture_'+this.bc_id](this);
                }
                else if (typeof SaviBcHandlers.enterpictureinpicture_default === 'function') {
                    SaviBcHandlers.enterpictureinpicture_default(this);
                }
            });
            this.video_player.on('leavepictureinpicture', () => {
                this.inPiPWindow = false;

                // call custom event handlers if defined
                if (typeof(SaviBcHandlers['leavepictureinpicture_'+this.bc_id]) === 'function') {
                    SaviBcHandlers['leavepictureinpicture_'+this.bc_id](this);
                }
                else if (typeof SaviBcHandlers.leavepictureinpicture_default === 'function') {
                    SaviBcHandlers.leavepictureinpicture_default(this);
                }
            });
        }
    }

    _add_ended_callback() {
        if (typeof(SaviBcHandlers['ended_'+this.bc_id]) === 'function') {
            this.video_player.on('ended', () => { 
                SaviBcHandlers['ended_'+this.bc_id](this);
            });
        }
        else if (typeof SaviBcHandlers.ended_default === 'function') {
            this.video_player.on('ended', () => { 
                SaviBcHandlers.ended_default(this);
            });
        }
    }

    _add_play_callback() {
        if (typeof(SaviBcHandlers['play_'+this.bc_id]) === 'function') {
            this.video_player.on('play', () => { 
                SaviBcHandlers['play_'+this.bc_id](this);
            });
        }
        else if (typeof SaviBcHandlers.play_default === 'function') {
           this.video_player.on('play', () => { 
                SaviBcHandlers.play_default(this);
            });
        }
    }

    _add_pause_callback() {
        if (typeof(SaviBcHandlers['pause_'+this.bc_id]) === 'function') {
            this.video_player.on('pause', () => { 
                SaviBcHandlers['pause_'+this.bc_id](this);
            });
        }
        else if (typeof SaviBcHandlers.pause_default === 'function') {
           this.video_player.on('pause', () => { 
                SaviBcHandlers.pause_default(this);
            });
        }
    }

    pause() {
        try {    
            this.video_player.pause();
        }
        catch(e) {
            safe_log('pause video ' + this.bc_id + ': ' + e.message);
        }
    }

    play() {
        try {
            this.video_player.play();
        }
        catch(e) {
            safe_log('play video ' + this.bc_id + ': ' + e.message);
        }
    }

    // status is either true or false
    mute(status=true) {
        if (status !== true) {
            status = false;
        }
        try {
            this.video_player.muted(status); 
        }
        catch(e) {
            safe_log('mute video ' + this.bc_id + ': ' + e.message);
        }
    }

    inPip() {
        return this.inPiPWindow;
    }
}

class SaviBcVideoCollection {

    constructor() {
        safe_log('Initializing SaviBcVideoCollection...');

        /*
        each item in video_collection is an object as follows:
        { section: section number, video: SaviBcVideo object }
        */
        this.video_collection = [];
        // add all videos in activity to video_collection
        this._get_videos_in_activity();
        // we need to wait for the videojs object to exist (this is created
        // by Brightcove) before we can add the handlers
        if (this.video_collection.length > 0) {
            this._wait_for_videojs();
        }
        else {
            safe_log('no videos found');
        }
    }

    // return an array of all video ids in collection
    getVideoIds() {
        const ids = [];
        this.video_collection.forEach(function(item) {
            ids.push(item.video.id);
        });
        return ids;
    }

    // return SaviBcVideo object for id
    getVideoObjById(id) {
        for (let i=0; i<this.video_collection.length; i++) {
            let vid_obj = this.video_collection[i].video;
            if (String(id) === vid_obj.id) {
                return vid_obj;
            }
        }
        return undefined;
    }

    // returns an array of all SaviBcVideo objects in collection
    getAllVideoObjs() {
        const video_objs = [];
        this.video_collection.forEach(function(item) {
            video_objs.push(item.video);
        });
        return video_objs;
    }

    // returns an array of all SaviBcVideo objects in sec_num
    getVideoObjsBySection(sec_num) {
        const video_objs = [];
        this.video_collection.forEach(function(item) {
            if (Number(sec_num) === item.section) {
                video_objs.push(item.video);
            }
        });
        return video_objs;
    }

    isVideoWatched(video_id) {
        return this.getVideoObjById(video_id).watched;
    }

    pauseAllVideos() {
        this.getAllVideoObjs().forEach(function(vid_obj) {
            vid_obj.pause();
        });
    }

    pauseAllVideosExceptPiP() {
        this.getAllVideoObjs().forEach(function(vid_obj) {
            if (!vid_obj.inPiPWindow) {
                vid_obj.pause();
            }
        });
    }

    // sec_num is optional. If sec_num is set to a section number then only
    // videos in that section will be checked. Otherwise, all videos in 
    // activity will be checked.
    allVideosWatched(sec_num) {
        sec_num = this._parse_sec_num(sec_num);
        let ALL_VIDEOS_WATCHED = true;
        const video_objs = (sec_num === undefined) ? this.getAllVideoObjs() : this.getVideoObjsBySection(sec_num);

        video_objs.forEach(function(vid_obj) {
            if (!vid_obj.watched) {
                ALL_VIDEOS_WATCHED = false;
            }
        });
       
        return ALL_VIDEOS_WATCHED;
    }
    
    _parse_sec_num(value) {
        if (typeof value === 'number') {
            if (!Number.isInteger(value) || value < 0) {
                return undefined;
            }
        }
        else if (typeof value === 'string') {
            if (value === "" || /[^0-9]/.test(value)) {
                return undefined;
            }
        }
        else {
            return undefined;
        }

        return Number(value);
    }

    // go through activity and find all SAVI videos and add them to 
    // the video_collection
    _get_videos_in_activity() {
        const $videos = $('[data-videoid]');
        const context = this;
        $videos.each(function() {
            const $video = $(this);
            const vid_id = $video.attr('data-videoid');
            const sec_num = $video.parents('[data-type="section"]').attr('id').replace('digfir_section_', '');
            context.video_collection.push({ section: Number(sec_num), video: new SaviBcVideo(vid_id)});
        });
    }

    // wait until the videojs object provided by brightcove is ready and 
    // then add custom handlers to the videos
    _wait_for_videojs() {
        const timer = setInterval(() => {
            if (typeof videojs === 'function' &&
              'players' in videojs) {
                let all_videos_loaded = true;
                this.getVideoIds().forEach(function(vid_id) {
                    if (videojs.players['player_'+vid_id] === undefined) {
                        all_videos_loaded = false;
                      }
                });
                if (all_videos_loaded) {
                  clearInterval(timer);
                  this._load_savi_handlers();
                }
              }
        }, 200); 
    }

    _load_savi_handlers() {
        this.video_collection.forEach(function(item) {
            item.video.add_video_player();
        });
        safe_log(this.video_collection.length + ' videos found');
    }

}
